import { Camera } from './camera';
import { EventHandler } from './eventhandler';
import { MouseEventProvider } from './mouseeventprovider';
import { Invalidate } from './renderer';
import { FirstPersonModifier } from './firstpersonmodifier';
import { TrackballModifier } from './trackballmodifier';
import { TurntableModifier } from './turntablemodifier';
/**
 * This navigation is merely a design template/recipe for more refined, specialized navigation and provides some basic,
 * commonly used camera modifier such as turntable, first-person, as well as trackball. This implementation is also
 * unfinished and will be continued as soon as possible (e.g., first-person navigation is not usable for now).
 * @todo - Refine and comment this class to be usable at least as common/most-basic navigation auxiliary.
 */
export declare class Navigation {
    /**
     * The navigation's invalidation callback. This should usually be setup by the owning renderer and invoke the
     * same callback the renderer has been given by the canvas. This invalidation is required, when continuous
     * rendering is not present, events might cause need for new rendering requests.
     */
    protected _invalidate: Invalidate;
    /** @see {@link camera} */
    protected _camera: Camera;
    /**
     * Currently active metaphor.
     */
    protected _metaphor: Navigation.Metaphor;
    /**
     * Identifies the active camera modifier.
     */
    protected _mode: Navigation.Modes | undefined;
    /**
     * Specifies, whether or not rotation mode should be invoked on any move event, regardless of buttons.
     */
    protected _alwaysRotateOnMove: boolean;
    /**
     * First person camera modifier.
     */
    protected _firstPerson: FirstPersonModifier | undefined;
    /**
     * Trackball camera modifier.
     */
    protected _trackball: TrackballModifier | undefined;
    /**
     * Turntable camera modifier.
     */
    protected _turntable: TurntableModifier | undefined;
    /**
     * Even handler used to forward/map events to specific camera modifiers.
     */
    protected _eventHandler: EventHandler;
    constructor(invalidate: Invalidate, mouseEventProvider: MouseEventProvider);
    /**
     * Resolves the event to camera modifier mapping by returning the responsible camera modifier.
     * @param event - Event to retrieve navigation mode for.
     */
    protected mode(event: MouseEvent | TouchEvent | KeyboardEvent): Navigation.Modes | undefined;
    protected rotate(event: MouseEvent | TouchEvent, start: boolean): void;
    protected onMouseDown(latests: Array<MouseEvent>, previous: Array<MouseEvent>): void;
    protected onMouseUp(latests: Array<MouseEvent>, previous: Array<MouseEvent>): void;
    protected onMouseMove(latests: Array<MouseEvent>, previous: Array<MouseEvent>): void;
    protected onClick(latests: Array<MouseEvent>, previous: Array<MouseEvent>): void;
    /**
     * Update should invoke navigation specific event processing. When using, e.g., an event handler, the event handlers
     * update method should be called in order to have navigation specific event processing invoked.
     */
    update(): void;
    /**
     * The camera that is to be modified in response to various events.
     */
    set camera(camera: Camera);
    /**
     * Configure this navigation's metaphor.
     */
    set metaphor(metaphor: Navigation.Metaphor);
    get metaphor(): Navigation.Metaphor;
}
export declare namespace Navigation {
    /**
     * Navigation modes used for identification of the current navigation intend, which is derived based on the event
     * types or gestures, regardless of the active navigation metaphor and its constraints.
     */
    enum Modes {
        Move = 0,
        Pan = 1,
        Rotate = 2,
        Zoom = 3,
        ZoomStep = 4
    }
    /**
     * Navigation metaphors supported by the default navigation implementation.
     */
    enum Metaphor {
        FirstPerson = "firstperson",
        Flight = "flight",
        Trackball = "trackball",
        Turntable = "turntable"
    }
}
