import { mat4, vec2 } from 'gl-matrix';
import { CameraModifier } from './cameramodifier';
/**
 * Math for camera rotation based on the trackball metaphor. The rotation computed by an initial (@see startRotate) and
 * subsequent (@see updateRotate) event points and can be applied to a camera via an explicit update (@see update).
 */
export declare class TrackballModifier extends CameraModifier {
    protected static readonly DEFAULT_SENSITIVITY = 0.002;
    /**
     * Current rotation matrix.
     */
    protected _rotation: mat4;
    /** @see {@link sensitivity} */
    protected _sensitivity: number;
    /**
     * Initiate a new trackball rotation at a specific event position.
     * @param point - Position of the current event to start the trackball rotation at.
     */
    initiate(point: vec2): void;
    /**
     * Update the trackball rotation w.r.t. a specific event position.
     * @param point - Position of the current event to continue/update the trackball rotation at.
     */
    process(point: vec2): void;
    /**
     * Actually applies the trackball rotation to the given camera.
     */
    update(): void;
    /**
     * Rotational sensitivity.
     */
    set sensitivity(sensitivity: number);
    get sensitivity(): number;
}
